module Dependency
  ( Action(..)
  , UToken(..)
  , Dependency(..)
  , makeDependency
  ) where

import Types.Token (Token(..))
import Types.Value (Value)
import Types.Function (Function)
import Types.FunctionIO (FunctionIO)

import Language.Haskell.TH.Syntax (Lift)

data Action = Function Function
            | FunctionIO FunctionIO
            | Inject Value
            | MapComp [Dependency]
            | FilterComp
            | GetListElem
            | SetListElem
  deriving (Show, Lift)

data UToken = UToken Int
            | UTupleToken UToken UToken
            | UZipToken UToken UToken
            | UNoToken
  deriving (Show, Lift)

data Dependency = Dependency UToken Action UToken
  deriving (Show, Lift)

makeDependency :: (Show a, Show b, Lift a, Lift b) => Token a -> Action -> Token b -> Dependency
makeDependency a action b = Dependency (makeUToken a) action (makeUToken b)

makeUToken :: Token a -> UToken
makeUToken = \case
  Token i -> UToken i
  TupleToken a b -> UTupleToken (makeUToken a) (makeUToken b)
  ZipToken a b -> UZipToken (makeUToken a) (makeUToken b)
  NoToken -> UNoToken

